unit uTestJMS;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls,
  jmsIntf, jmsException, jmsSonicMQ, apiSonicMQ;

type
  TfrmTestJMS = class(TForm, IMessageListener)
    gbSenderAndReceive: TGroupBox;
    eHost: TEdit;
    btConnect: TButton;
    btDisconnect: TButton;
    eQueue: TEdit;
    btSendMessage: TButton;
    btReceiveMessage: TButton;
    btCreateAsyncReceiver: TButton;
    btCloseAsyncReceiver: TButton;
    GroupBox1: TGroupBox;
    eTopicHost: TEdit;
    btTopicConnect: TButton;
    btTopicDisconnect: TButton;
    eTopic: TEdit;
    btPublishMessage: TButton;
    btCreateAsyncSubscriber: TButton;
    btCloseAsyncSubscriber: TButton;
    lbLogs: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    eMessage: TEdit;
    ePublished: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    procedure btConnectClick(Sender: TObject);
    procedure btDisconnectClick(Sender: TObject);
    procedure btSendMessageClick(Sender: TObject);
    procedure btReceiveMessageClick(Sender: TObject);
    procedure btCreateAsyncReceiverClick(Sender: TObject);
    procedure btCloseAsyncReceiverClick(Sender: TObject);
    procedure btTopicConnectClick(Sender: TObject);
    procedure btTopicDisconnectClick(Sender: TObject);
    procedure btPublishMessageClick(Sender: TObject);
    procedure btCreateAsyncSubscriberClick(Sender: TObject);
    procedure btCloseAsyncSubscriberClick(Sender: TObject);
  private
    factory : IQueueConnectionFactory;
    qConnection : IQueueConnection;
    qSession : IQueueSession;
    queue : IQueue;
    receiver : IQueueReceiver;

    topicFactory : ITopicConnectionFactory;
    topicConnection : ITopicConnection;
    topicSession : ITopicSession;
    topic : ITopic;
    subscriber : ITopicSubscriber;

    procedure DoLog (msg : string);

    procedure onMessage(message : IMessage);
  public
    { Public declarations }
  end;

var
  frmTestJMS: TfrmTestJMS;

implementation

{$R *.dfm}

procedure TfrmTestJMS.btConnectClick(Sender: TObject);
begin
  DoLog('Connecting...');
  factory := TSMQQueueConnectionFactory.Create(eHost.Text);
  qConnection := factory.createQueueConnection;
  qConnection.start;
  qSession := qConnection.createQueueSession(false, Session_AUTO_ACKNOWLEDGE);
  queue := qSession.createQueue(eQueue.Text);

  btConnect.Enabled := False;
  btDisconnect.Enabled := True;

  btSendMessage.Enabled := True;
  btReceiveMessage.Enabled := True;
  btCreateAsyncReceiver.Enabled := True;

  DoLog('Success');
end;

procedure TfrmTestJMS.btDisconnectClick(Sender: TObject);
begin
  DoLog('Disconnecting...');
  if Assigned (receiver) then
  begin
    receiver.close;
    receiver := Nil;
  end;
  if Assigned (queue) then
  begin
    queue := Nil;
  end;
  if Assigned (qSession) then
  begin
    qSession.close;
    qSession := Nil;
  end;
  if Assigned (qConnection) then
  begin
    qConnection.close;
    qConnection := Nil;
  end;
  if Assigned (factory) then
  begin
    factory := Nil;
  end;

  btConnect.Enabled := True;
  btDisconnect.Enabled := False;

  btSendMessage.Enabled := False;
  btReceiveMessage.Enabled := False;
  btCreateAsyncReceiver.Enabled := False;
  btCloseAsyncReceiver.Enabled := False;

  DoLog('Success');
end;

procedure TfrmTestJMS.DoLog(msg: string);
begin
  lbLogs.Items.Add(msg);
  lbLogs.ItemIndex := lbLogs.Items.Count-1;
end;

procedure TfrmTestJMS.btSendMessageClick(Sender: TObject);
var
  snd : IQueueSender;
  msg : ITextMessage;
begin
  DoLog('Sending...');
  snd := qSession.createSender(queue);
  try
    msg := qSession.createTextMessage;
    msg.Text := eMessage.Text;
    snd.send(msg);
  finally
    snd.close;
  end;

  DoLog('Success');
end;

procedure TfrmTestJMS.btReceiveMessageClick(Sender: TObject);
var
  rcv : IQueueReceiver;

  msg : IMessage;
  msgText : ITextMessage;
begin
  DoLog('Receiving...');
  rcv := qSession.createReceiver(queue);
  try
    btCreateAsyncReceiver.Enabled := False;
    msg := rcv.receiveNoWait;
    if Assigned (msg) then
    begin
      DoLog ('Success');
      try
        msgText := msg as ITextMessage;
        DoLog ('Receive text message: '+msgText.Text);
      except
        DoLog ('Receive unsupported JMS message type');
      end;
    end
    else
      DoLog ('Nothing to receive');
  finally
    rcv.close;
  end;
end;

procedure TfrmTestJMS.onMessage(message: IMessage);
var
  msgText : ITextMessage;
begin
  DoLog('Receive message');
  try
    msgText := message as ITextMessage;
    DoLog ('Receive text message: '+msgText.Text);
  except
    DoLog ('Receive unsupported JMS message type');
  end;
end;

procedure TfrmTestJMS.btCreateAsyncReceiverClick(Sender: TObject);
begin
  DoLog('Create async receiver');

  receiver := qSession.createReceiver(queue);
  receiver.MessageListener := Self;

  btReceiveMessage.Enabled := False;
  btCreateAsyncReceiver.Enabled := False;
  btCloseAsyncReceiver.Enabled := True;
  DoLog('Success')
end;

procedure TfrmTestJMS.btCloseAsyncReceiverClick(Sender: TObject);
begin
  DoLog('Close async receiver');

  receiver.MessageListener := Nil;
  receiver.close;
  receiver := nil;

  btCreateAsyncReceiver.Enabled := True;
  btCloseAsyncReceiver.Enabled := False;
  DoLog('Success');
end;

procedure TfrmTestJMS.btTopicConnectClick(Sender: TObject);
begin
  DoLog('Connecting...');
  topicFactory := TSMQTopicConnectionFactory.Create(eTopicHost.Text);
  topicConnection := topicFactory.createTopicConnection;
  topicConnection.start;
  topicSession := topicConnection.createTopicSession(false, Session_AUTO_ACKNOWLEDGE);
  topic := topicSession.createTopic(eTopic.Text);

  btTopicConnect.Enabled := False;
  btTopicDisconnect.Enabled := True;

  btPublishMessage.Enabled := True;
  btCreateAsyncSubscriber.Enabled := True;

  DoLog('Success');
end;

procedure TfrmTestJMS.btTopicDisconnectClick(Sender: TObject);
begin
  DoLog('Disconnecting...');
  if Assigned (subscriber) then
  begin
    subscriber.close;
    subscriber := Nil;
  end;
  if Assigned (topic) then
  begin
    topic := Nil;
  end;
  if Assigned (topicSession) then
  begin
    topicSession.close;
    topicSession := Nil;
  end;
  if Assigned (topicConnection) then
  begin
    topicConnection.close;
    topicConnection := Nil;
  end;
  if Assigned (topicFactory) then
  begin
    topicFactory := Nil;
  end;

  btTopicConnect.Enabled := True;
  btTopicDisconnect.Enabled := False;

  btPublishMessage.Enabled := False;
  btCreateAsyncSubscriber.Enabled := False;
  btCloseAsyncSubscriber.Enabled := False;

  DoLog('Success');
end;

procedure TfrmTestJMS.btPublishMessageClick(Sender: TObject);
var
  pbl : ITopicPublisher;
  msg : ITextMessage;
begin
  DoLog('Publishing...');
  pbl := topicSession.createPublisher(topic);
  try
    msg := topicSession.createTextMessage;
    msg.Text := ePublished.Text;
    pbl.publish(msg);
  finally
    pbl.close;
  end;

  DoLog('Success');
end;

procedure TfrmTestJMS.btCreateAsyncSubscriberClick(Sender: TObject);
begin
  DoLog('Create async subscriber');

  subscriber := topicSession.createSubscriber(topic);
  subscriber.MessageListener := Self;

  btCreateAsyncSubscriber.Enabled := False;
  btCloseAsyncSubscriber.Enabled := True;
  DoLog('Success')
end;

procedure TfrmTestJMS.btCloseAsyncSubscriberClick(Sender: TObject);
begin
  DoLog('Close async receiver');

  subscriber.MessageListener := Nil;
  subscriber.close;
  subscriber := nil;

  btCreateAsyncSubscriber.Enabled := True;
  btCloseAsyncSubscriber.Enabled := False;
  DoLog('Success');
end;

end.

